/*------------------------------------------------------------------------------*

	main.c

	PowerPC EABI C Project Stationery.

 *------------------------------------------------------------------------------*/
/*------------
I/O Registers
------------*/
short Com1Con0:0x305008;	// SCI1 control register 0
short Com1Con1:0x30500A;	// SCI1 control register 1
short Com1Status:0x30500C;	// SCI1 status register
short Com1Data:0x30500E;	// SCI1 data register

short WatchDog:0x2FC00E;	// SWSR software service register

/*------------------
Function Prototypes
------------------*/
void ClearWatchdog();
char putchar(unsigned char sbyte);
char ReadyToSend();
void puts(char *sptr);
void putch(char ch);
char putchar(unsigned char sbyte);
char SerialReady();
char GetSByte();
void InitCom1();

unsigned char SByte;	// global byte, contained just received data byte

void main()
{
	InitCom1();	// initialize SCI1 serial port
	for( ; ;){
		puts("\n\nHello World\n");
		GetSByte();	// wait for data to be received
	}
}

void InitCom1(){
//	Com1Con0 = 130;		// 9600 baud with 40mh internal clock	(pll reg - 0x09)  (0-11 value)
	Com1Con0 = 65;		// 9600 baud with 20mh internal clock 	(pll reg - 0x04)  (0-11 value)
//	Com1Con0 = 13;		// 9600 baud with 4mh internal clock
	Com1Con1 = 0x000C;	// Transmit & Receive Enabled, no parity, no interrupts
}

char putchar(unsigned char sbyte){
	if(ReadyToSend())	return 1;	// wait for ready to send
	Com1Data = (short) sbyte;
	return 0;
}

void putch(char ch){

//	if (ch == '\n'){
//		putchar(CR);
//		putchar(LF);
//	}
//	else{
		putchar(ch);
//	}
}

void puts(char *sptr){
	while(*sptr){
		putch(*sptr);
		++sptr;
	}
}

// reset / clear the watchdog
void ClearWatchdog(){
	WatchDog = 0x556C;	// clear the watchdog
	WatchDog = 0xAA39;
}

// returns 0 if transmit register ready for another byte
char ReadyToSend(){
	short Status;
	do{	// wait for transmit byte sent to the shift register, ready for another
		Status = Com1Status;
		ClearWatchdog();	// feed the dog
	}while((Status & 0x0100) == 0);
	return 0;
}
// returns 0 if no serial data is waiting to be received
char SerialReady(){
	short Status;

	ClearWatchdog();	// feed the dog
	Status = Com1Status;
	if( (Status & 0x0040) == 0){
		return 1;
	}
	return 0;
}
//---------------------------------------------------------------------------
// wait for a byte to be received from the serial port.
// the byte will be stored in the global register:	SByte
char GetSByte(){
	short Temp1;

	while(SerialReady() == 1) ;	// wait for new incomming data

	Temp1 = Com1Data; 	// read the port
	SByte = (unsigned char) (Temp1 & 0x00FF);
	return 0;
}

//---------------------------------------------------------------------------
// INITIALIZE HARDWARE REGISTERS
// This routine is called once following RESET
//void inithw(){
void usr_init(){
	unsigned long *Rptr;
	unsigned short *sRptr;
/*
@ SPR638 = 0xFFF00800	<--- see __ppc_eabi_init.c
@ SPR560 = 0x00000000

*/

	Rptr = (unsigned long *) 0x2fc000;	// SIUMCR
	*Rptr = 0x00;
	Rptr = (unsigned long *) 0x2fc004;	// SYPCR
	*Rptr = 0xFF88;

	Rptr = (unsigned long *) 0x2fc100;	// BR0
	*Rptr = 0x400003;	// 32-bit set here for BDM or external flash
//	*Rptr = 0x400803;	// 16-bit set here for EEPROM M-SEL position 1 (cs0)
//	*Rptr = 0x000003;	// 32-bit set here to use external booting with IMMR Flash disabled
	Rptr = (unsigned long *) 0x2fc104;	// OR0
	*Rptr = 0xfff80012;

	Rptr = (unsigned long *) 0x2fc108;	// BR1
	*Rptr = 0x800003;
	Rptr = (unsigned long *) 0x2fc10c;	// OR1
	*Rptr = 0xfff80002;	// no wait state

	Rptr = (unsigned long *) 0x2fc110;	// BR2
	*Rptr = 0xc00003;
	Rptr = (unsigned long *) 0x2fc114;	// OR2
	*Rptr = 0xfffe0022;

	Rptr = (unsigned long *) 0x2fc118;	// BR3
	*Rptr = 0x01000403;	// works for Keypad/LCD
	Rptr = (unsigned long *) 0x2fc11c;	// OR3
	*Rptr = 0xfffe0FF1;	// maximum wait states

	Rptr = (unsigned long *) 0x2fc140;	// DMBR
	*Rptr = 0x00;
	Rptr = (unsigned long *) 0x2fc144;	// DMOR
	*Rptr = 0x00;
	Rptr = (unsigned long *) 0x2fc280;	// SCCR
	*Rptr = 0x10000;
	Rptr = (unsigned long *) 0x2fc284;	// PLPRCR
	*Rptr = 0x400000;

	sRptr = (unsigned short *) 0x300000;	// DPT
	*sRptr = 0x00;

	sRptr = (unsigned short *) 0x300004;	// RAM base addr
	*sRptr = 0xffa0;

	sRptr = (unsigned short *) 0x305014;	// PORTQS
	*sRptr = 0x00;
	sRptr = (unsigned short *) 0x305016;	// PQSPAR/DDRQS
	*sRptr = 0x00;
	sRptr = (unsigned short *) 0x306100;	// MPIOSMDR
	*sRptr = 0x00;
	sRptr = (unsigned short *) 0x306102;	// MPIOSMDDR
	*sRptr = 0x00;
	sRptr = (unsigned short *) 0x306800;	// MPIOS1TPCR
	*sRptr = 0x00;
	sRptr = (unsigned short *) 0x380000;	// SRAMMCR
	*sRptr = 0x0000;

	Rptr = (unsigned long *) 0x2fc024;	// SGPIODT1
	*Rptr = 0x0000;
	Rptr = (unsigned long *) 0x2fc028;	// SGPIODT2
	*Rptr = 0x0000;
	Rptr = (unsigned long *) 0x2fc02c;	// SGPIOCR
	*Rptr = 0x0000;
	Rptr = (unsigned long *) 0x2fc030;	// EMCR
	*Rptr = 0x0000;
	Rptr = (unsigned long *) 0x307f80;	// UMCR
	*Rptr = 0x0000;
}

void InterruptHandler(unsigned long exceptNumber){
	puts("\nException Interrupt Handled by User Flash Code");
}

